from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC



class DragAndDrop(customwebDriverwait):
    # drag and drop
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
        '''
        drag_and_drop(source, target)¶
        Holds down the left mouse button on the source element,
        then moves to the target element and releases the mouse button.
        Args:	
        source: The element to mouse down.
        target: The element to mouse up
        '''
    def dragAndDrop(self, sourcexpath, destinationxpath):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            DestinationElementPresense=super().WaitFor_PresenseOf_Element_Located(destinationxpath)
            DestinationElementVisibility=super().WaitFor_VisibilityOf_Element_Located(destinationxpath)
            if(SourceelElementPresense & DestinationElementPresense ):
                if(SourceelElementVisibility & DestinationElementVisibility):
                    sourceElement = self.driver.find_element(By.XPATH, sourcexpath)
                    destination = self.driver.find_element(By.XPATH, destinationxpath)
                    action_chains = ActionChains(self.driver)
                    action_chains.drag_and_drop(sourceElement, destination).perform()
        except Exception as error:
            raise error            
        '''
        drag_and_drop_by_offset(source, xoffset, yoffset)¶
        Holds down the left mouse button on the source element,
        then moves to the target offset and releases the mouse button.
        Args:	
        source: The element to mouse down.
        xoffset: X offset to move to.
        yoffset: Y offset to move to.
        '''
    def dragAndDropByOffset(self, sourcexpath, xoffset, yoffset):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            if(SourceelElementPresense):
                if(SourceelElementVisibility):
                    sourceElement = self.driver.find_element(By.XPATH, sourcexpath)

                    action_chains = ActionChains(self.driver)
                    action_chains.drag_and_drop_by_offset(
                        sourceElement, xoffset, yoffset).perform()
        except Exception as error:
            raise error                


from pyallied.web.webWaits import customwebDriverwait
class DropDownActions(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
    def Uninstall_addon(self,identifier):
        if(self.driver.capabilities['browser_name']=="firefox"):
            self.driver.uninstall_addon(identifier)
        else:
            raise "add option is allowed only on firefox"
    '''
    Installs Firefox addon.

    Returns identifier of installed addon. This identifier can later be used to uninstall addon.

    Parameters:	
    path – Absolute path to the addon that will be installed.

    Usage:	
    driver.install_addon('/path/to/firebug.xpi')
    '''
    def Install_addon(self,path, temporary=False):
        if(self.driver.capabilities['browser_name']=="firefox"):
            self.driver.install_addon(path)
        else:
            raise "install addon is allowed only on firefox"
    
    def Set_context(self,context):
        if(self.driver.capabilities['browser_name']=="firefox"):
            self.driver.set_context(context)
        else:
            raise "set context is allowed only on firefox"
    def Get_firefox_profile(self,context):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile
        else:
            raise "get firefox profile is allowed only on firefox"
    '''def is_headless(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.headless
        else:
            raise "is headless? is allowed only on firefox" '''
    def Add_extension(self,extension='webdriver.xpi'):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.add_extension(extension)
        else:
            raise "Add_extension is allowed only on firefox" 
    '''sets the preference that we want in the profile.'''        
    def Set_preference(self,key, value):
        if(self.driver.capabilities['browser_name']=="firefox"):
            self.driver.firefox_profile.set_preference(key, value)
        else:
            raise "Set_preference is allowed only on firefox"
    '''sets the preference that we want in the profile.'''        
    def get_default_capabilities(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.default_capabilities
        else:
            raise "get_default_capabilities is allowed only on firefox"
    def Update_capabilities(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.update_preferences()
        else:
            raise "get_default_capabilities is allowed only on firefox"                          
    def get_binary_instance(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.binary()
        else:
            raise "binary is allowed only on firefox"   
    def Set_accept_untrusted_certs(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.accept_untrusted_certs
        else:
            raise "accept_untrusted_certs is allowed only on firefox"
    def Set_assume_untrusted_cert_issuer(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.assume_untrusted_cert_issuer
        else:
            raise "assume_untrusted_cert_issuer is allowed only on firefox"
    def Get_encoded_Profile(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.encoded
        else:
            raise "Get_encoded_Profile is allowed only on firefox"
    def Get_Profile_path(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.path
        else:
            raise "path is allowed only on firefox"
    def Get_Profile_DEFAULT_PREFERENCES(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.DEFAULT_PREFERENCES 
        else:
            raise "DEFAULT_PREFERENCES  is allowed only on firefox"
    def Get_Profile_ANONYMOUS_PROFILE_NAME(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.ANONYMOUS_PROFILE_NAME 
        else:
            raise "ANONYMOUS_PROFILE_NAME is allowed only on firefox"
    def Get_Profile_port(self):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.firefox_profile.port 
        else:
            raise "get port is allowed only on firefox" 
    def Add_command_line_options(self,*args):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.binary.add_command_line_options(*args) 
        else:
            raise "add_command_line_options is allowed only on firefox"
    def kill_Binary_instance(self,*args):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.binary.kill() 
        else:
            raise "kill_Binary_instance is allowed only on firefox"
    def Launch_browser(self,profile, timeout=30):
        if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.binary.launch_browser(profile, timeout=30) 
        else:
            raise "Launch_browser is allowed only on firefox"                                                                                                                                                                                                                 


from select import select
from xml.dom.minidom import Element
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.actions.wheel_input import ScrollOrigin


class common(customwebDriverwait):

    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    def findElementsBy(self, xpath):
        try:
            if(super().WaitFor_presence_of_all_elements_located(xpath)):
                return self.driver.find_elements(By.XPATH, xpath)
        except Exception as error:
            raise error

        # self.driver.presence_of_all_elements_located(xpath)
        # self.driver.visibility_of_element_located(xpath)
        
        

    def findElementBy(self, xpath):
        try:
            self.driver.presence_of_element_located(xpath)
            self.driver.visibility_of_element_located(xpath)
            if(super().WaitFor_PresenseOf_Element_Located(xpath)) :
                self.driver.find_element(By.XPATH, xpath)
        except Exception as error:
            raise error        

    def click(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    self.driver.find_element(By.XPATH, xpath).click()
        except Exception as error:
            raise error            

    def clear(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    self.driver.find_element(By.XPATH, xpath).clear()  
        except Exception as error:
            raise error              
    
    
    def isClickable(self, xpath, index=None):
        # self.driver.visibility_of_element_located(xpath)
        try:
            return WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((xpath)))
        except Exception as error:
            raise error    
    
    '''
    Simulates typing into the element.

    Args:	
    value - A string for typing, or setting form fields. For setting file inputs, this could be a local file path.
    Use this to send simple key events or to fill out form fields:
    form_textfield = driver.find_element(By.NAME, 'username')
                    form_textfield.send_keys("admin")
    This can also be used to set file inputs.

    file_input = driver.find_element(By.NAME, 'profilePic')
    file_input.send_keys("path/to/profilepic.gif")
    # Generally it's better to wrap the file path in one of the methods
    # in os.path to return the actual path to support cross OS testing.
    # file_input.send_keys(os.path.abspath("path/to/profilepic.gif"))
    '''

    def fillField(self, xpath, text):
        try:
            elementPresence = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisibility = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresence):
                if(elementVisibility):
                    self.driver.find_element(By.XPATH, xpath).send_keys(text)
        except Exception as error:
            raise error

    def navigateto(self, urlstring):
        try:
            self.driver.get(urlstring)
        except Exception as error:
            raise error
    '''
        Gets the given attribute or property of the element.

        This method will first try to return the value of a property with the given name. If a property with that name doesn’t exist, it returns the value of the attribute with the same name. If there’s no attribute with that name, None is returned.

        Values which are considered truthy, that is equals “true” or “false”, are returned as booleans. All other non-None values are returned as strings. For attributes or properties which do not exist, None is returned.

        To obtain the exact value of the attribute or property, use get_dom_attribute() or get_property() methods respectively.

        Args:	
        name - Name of the attribute/property to retrieve.
        Example:

        # Check if the "active" CSS class is applied to an element.
        is_active = "active" in target_element.get_attribute("class")
    '''
    def getAttribute(self, xpath,AttributeName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).get_attribute(AttributeName)
        except Exception as error:
            raise error                  
    
    '''
        Gets the given attribute of the element. Unlike get_attribute(), this method only returns 
        attributes declared in the element’s HTML markup.

        Args:	
        name - Name of the attribute to retrieve.
        Usage:	
        text_length = target_element.get_dom_attribute("class")
    '''
    def getDomAttribute(self, xpath,AttributeName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).get_dom_attribute(AttributeName) 
        except Exception as error:
            raise error


    '''
        Gets the given property of the element.

        Args:	
        name - Name of the property to retrieve.
        Usage:	
        text_length = target_element.get_property("text_length")
    '''
    def getProperty(self, xpath,PropertyName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).get_property(PropertyName)
        except Exception as error:
            raise error             
    '''
        Whether the element is visible to a user.
    '''
    def isElementDisplayed(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).is_displayed()
        except Exception as error:
            raise error             
    '''
        Returns whether the element is enabled.
    '''
    def isElementEnabled(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).is_enabled()  
        except Exception as error:
            raise error
    '''
        Returns whether the element is selected.

        Can be used to check if a checkbox or radio button is selected.
    '''
    def isElementSelected(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).is_selected() 
        except Exception as error:
            raise error
    '''
        Saves a screenshot of the current element to a PNG image file. Returns
        False if there is any IOError, else returns True. Use full paths in your filename.
    '''
    def currentElementScreenshot(self, xpath,screenShotSavingPath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return self.driver.find_element(By.XPATH, xpath).screenshot(screenShotSavingPath)
        except Exception as error:
            raise error
        

    # action class

    def actionClick(self, xpath):
        try:
            actions = ActionChains(self.driver)
            
            if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                element = self.driver.find_element(By.XPATH, xpath)
                actions.move_to_element(element).click(element)
                actions.perform()
        except Exception as error:
            raise  error    

        '''
        Holds down the left mouse button on an element.

        Args:	
        on_element: The element to mouse down. If None, clicks on current mouse position.
        '''
    def actionClickandHold(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
            else:
                element = None
            if(element != None):
                actions.click_and_hold(element)
                actions.perform()
            else:
                actions.click_and_hold()
                actions.perform()
        except Exception as error:
            raise error        
    '''
    Moves the mouse to the element
    '''
    def moveToElement(self, xpath):
        try:
            actions = ActionChains(self.driver)
            if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                element = self.driver.find_element(By.XPATH, xpath)
                actions.move_to_element(element).perform()
        except Exception as error:
            raise error
                
    '''
    Moves the mouse to the element and clicks it
    '''
    def moveToElement_and_click(self, xpath):
        try:
            actions = ActionChains(self.driver)
            if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                element = self.driver.find_element(By.XPATH, xpath)
                actions.move_to_element(element).click(element).perform()
        except Exception as error:
            raise error
        '''
        if both xpathOfMovingTO and xpathOfElemtToClick are provided, it will click the subemenet like below example
        ex:
        actions = ActionChains(driver)
        actions.move_to_element(menu)
        actions.click(hidden_submenu)
        actions.perform()
        if user didnt provide subelement xpath, then it clicks the same element mouse moved to
        '''    
    def moveToElement_and_subElement_click(self, xpathOfMovingTO, xpathOfElemtToClick=None):
        try:
            if(xpathOfElemtToClick!=None):    
                actions = ActionChains(self.driver)
                if(super().WaitFor_PresenseOf_Element_Located(xpathOfMovingTO)):
                    MovedToelement = self.driver.find_element(By.XPATH, xpathOfMovingTO)
                    ClickElement=  self.driver.find_element(By.XPATH, xpathOfElemtToClick)  
                    actions.move_to_element(MovedToelement).click(ClickElement).perform()
            else:
                if(super().WaitFor_PresenseOf_Element_Located(xpathOfMovingTO)):
                    MovedToelement = self.driver.find_element(By.XPATH, xpathOfMovingTO)
                      
                    actions.move_to_element(MovedToelement).click().perform()       
        except Exception as error:
            raise error 
            '''
            Move the mouse by an offset of the specified element.
            Offsets are relative to the top-left corner of the element.
            Args:	
            to_element: The WebElement to move to.
            xoffset: X offset to move to.
            yoffset: Y offset to move to.
            '''           
    def moveToElementWithOffset(self, xpath, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                element = self.driver.find_element(By.XPATH, xpath)
                actions.move_to_element_with_offset(element, xoffset, yoffset)
                actions.perform()
        except Exception as error:
            raise error    
    def moveByOffsett(self, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            actions.move_by_offset(xoffset, yoffset)
            actions.perform()
        except Exception as error:
            raise error    
    def actionRelease(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
            else:
                element = None
            if(element != None):
                actions.release(element)
                actions.perform()

            else:
                actions.release()
                actions.perform()
        except Exception as error:
            raise error        
    

    def keyDown_and_sendKeys(self, ModifierKey, keys, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
                    actions.key_down(ModifierKey, element).send_keys(keys)
                    actions.perform()                    
            else:
                actions.key_down(ModifierKey).send_keys(keys)
                actions.perform()
        except Exception as error:
            raise error

    '''
    Releases a modifier key.

    Args:	
    value: The modifier key to send. Values are defined in Keys class.
    element: The element to send keys. If None, sends a key to current focused element.
    Example, pressing ctrl+c:

    ActionChains(driver).key_down(Keys.CONTROL).send_keys('c').key_up(Keys.CONTROL).perform()
    
    '''
    def keyUp(self, ModifierKey, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):    
                    element = self.driver.find_element(By.XPATH, xpath)
                    actions.key_up(ModifierKey, element).perform()
            else:

                actions.key_up(ModifierKey).perform()
        except Exception as error:
            raise error  
        '''
         keyboard key press down --> keyboard send keys -->keyboard releasing the key 
        '''
    def keyDown_sendKeys_keyUP(self,ModifierKey,keys:str, xpath=None):
        try:
            actions = ActionChains(self.driver)   
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):    
                    element = self.driver.find_element(By.XPATH, xpath)
                    actions.key_down(ModifierKey,element).send_keys(keys).key_up(ModifierKey,element).perform()
            else:
                actions.key_down(ModifierKey).send_keys(keys).key_up(ModifierKey).perform()
        except Exception as error:
            raise error    



    '''
    Double-clicks an element.

    Args:	
    on_element: The element to double-click. If None, clicks on current mouse position.
    '''
    def double_click(self, xpath: None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
                    actions.double_click(element)
                    actions.perform()

            else:
                actions.double_click()
                actions.perform()
        except Exception as error:
            raise error        

        '''
        Performs a context-click (right click) on an element.

        Args:	
        on_element: The element to context-click. If None, clicks on current mouse position.
        '''
    def right_Click(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
            else:
                element = None
            if(element != None):
                actions.context_click(element)
                actions.perform()
            else:
                actions.context_click()
                actions.perform() 
        except Exception as error:
            raise error               

    def resetActions(self):
        try:
            actions = ActionChains(self.driver)
            actions.reset_actions()
            actions.perform()
        except Exception as error:
            raise error            

    def sendKeys(self, *keys_to_send):
        try:
            actions = ActionChains(self.driver)
            actions.send_keys(keys_to_send)
            actions.perform()
        except Exception as error:
            raise error            

    def sendKeysToElement(self, xpath, *keys_to_send):
        try:
            actions = ActionChains(self.driver)
            element = super().WaitFor_PresenseOf_Element_Located(xpath)
            actions.send_keys_to_element(element, keys_to_send)
            actions.perform()
        except Exception as error:
            raise error    

        '''
        Sends wheel scroll information to the driver to be processed.

        Args:	
        x: starting X coordinate
        y: starting Y coordinate
        delta_x: the distance the mouse will scroll on the x axis
        delta_y: the distance the mouse will scroll on the y axis

        '''
    def Scroll(self,x: int, y: int, delta_x: int, delta_y: int, duration: int = 0, origin: str = 'viewport'):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll(x, y, delta_x, delta_y, duration, origin)
            actions.perform()
        except Exception as error:
            raise error 

        '''
        Scrolls by provided amounts with the origin in the top left corner of the viewport.

    Args:	
    delta_x: Distance along X axis to scroll using the wheel. A negative value scrolls left.
    delta_y: Distance along Y axis to scroll using the wheel. A negative value scrolls up.
        '''
    def Scroll_by_amount(self,delta_x: int, delta_y: int):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll_by_amount( delta_x, delta_y)
            actions.perform()
        except Exception as error:
            raise error
        '''
        Scrolls by provided amount based on a provided origin. The scroll origin is either the center of an 
        element or the upper left of the viewport plus any offsets. If the origin is an element, 
        and the element is not in the viewport, the bottom of the element will first be scrolled to the bottom of the viewport.

        Args:	
        origin: Where scroll originates (viewport or element center) plus provided offsets.
        delta_x: Distance along X axis to scroll using the wheel. A negative value scrolls left.
        delta_y: Distance along Y axis to scroll using the wheel. A negative value scrolls up.
        Raises:	If the origin with offset is outside the viewport. - MoveTargetOutOfBoundsException - If the origin with offset 
        is outside the viewport.
        '''
    def Scroll_from_origin(self,scroll_origin: ScrollOrigin,delta_x: int, delta_y: int):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll_from_origin( scroll_origin,delta_x, delta_y)
            actions.perform()
        except Exception as error:
            raise error


        '''
                If the element is outside the viewport, scrolls the bottom of the element to the bottom of the viewport.

        Args:	
        element: Which element to scroll into the viewport.
        send_keys(*keys_to_send)
        Sends keys to current focused element.

        Args:	
        keys_to_send: The keys to send. Modifier keys constants can be found in the ‘Keys’ class.
        '''
    def scroll_to_Element(self, xpath):
        try:
            actions = ActionChains(self.driver)
            if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                element = self.driver.find_element(By.XPATH, xpath)
                actions.scroll_to_element( element)
                actions.perform()
        except Exception as error:
            raise error
    def switch_To_ActiveElement(self):
        try:

            self.driver.switch_to.active_element
        except Exception as error:
            raise error
    def formSubmit(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            element.submit()
        except Exception as error:
            raise error
    def get_Value_of_css_property(self,xpath,property_name):
        try:
            element=self.findElementBy(xpath) 
            return element.value_of_css_property(property_name)
        except Exception as error:
            raise error
    def get_Accessible_name(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.accessible_name
        except Exception as error:
            raise error 
    def get_Aria_role(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.Aria_role
        except Exception as error:
            raise error 
    '''Internal ID used by selenium.

    Internal ID used by selenium.

    This is mainly for internal use. Simple use cases such as checking if 2 webelements refer to the same element, can be done using ==:

    if element1 == element2:
        print("These 2 are equal")
    '''                                                  
    def Get_internal_ID(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.id
        except Exception as error:
            raise error
    def Get_location_of_Element(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.location
        except Exception as error:
            raise error
    '''
    THIS PROPERTY MAY CHANGE WITHOUT WARNING. Use this to discover where on the screen an element is 
    so that we can click it. This method should cause the element to be scrolled into view.

    Returns the top lefthand corner location on the screen, or None if the element is not visible.
    '''
    def get_Scroll_location_of_Element(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.location_once_scrolled_into_view
        except Exception as error:
            raise error
    '''
    Internal reference to the WebDriver instance this element was found from.
    '''        
    def get_Prent_of_Element(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.parent
        except Exception as error:
            raise error
    '''
    A dictionary with the size and location of the element.
    '''        
    def get_size_And_Location(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.rect
        except Exception as error:
            raise error
    def get_element_screenshot_as_base64(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.screenshot_as_base64
        except Exception as error:
            raise error
    def get_screenshot_as_png(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.screenshot_as_png
        except Exception as error:
            raise error
    '''
    shadow_root¶
    Returns a shadow root of the element if there is one or an error. Only works from Chromium 96 onwards. Previous versions of Chromium based browsers will throw an assertion exception.

    Returns:	
    ShadowRoot object or
    NoSuchShadowRoot - if no shadow root was attached to element
    
    '''        
    def get_Element_shadow_root(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.shadow_root
        except Exception as error:
            raise error
    def get_Element_size(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.size
        except Exception as error:
            raise error
    def get_Element_tag_name(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.tag_name
        except Exception as error:
            raise error
    def get_Element_text(self,xpath):
        try:
            element=self.findElementBy(xpath) 
            return element.text
        except Exception as error:
            raise error                                                                                                  


from typing import Dict
from selenium.webdriver.common.utils import is_url_connectable
from selenium import webdriver
from appium.webdriver.common.appiumby import AppiumBy
from appium.protocols.webdriver.can_execute_scripts import CanExecuteScripts
import polling2
import time
from appium.webdriver.common.appiumby import AppiumBy
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.common.exceptions import ElementNotVisibleException


class mobAppium:
    def __init__(self, driver):
        self.driver = driver
        '''
        Returns information about whether a remote end is in a state in which it can create new sessions 
        and can additionally include arbitrary meta information that is specific to the implementation.
        usage: selenium.webdriver.common.utils.is_url_connectable(port)


        '''

    def get_Remote_Driver_Status(self, port):
        self.driver.is_url_connectable(port)

        '''
        Execute a variety of native, mobile commands that aren't associated with a specific endpoint

        Syntax is execute("mobile: <commandName>", <JSON serializable argument>)
        
        useage: self.driver.execute_script("mobile: scroll", {'direction': 'down'})


        '''

    def execute_Mobile_Command(self, Mobilecommand: str, parameters: Dict[str, any]):
        self.driver.execute_script(Mobilecommand, parameters)
        '''
        The server should attempt to create a session that most closely matches the desired and required capabilities.

        JSONWP Spec Required capabilities have higher priority than desired 
        capabilities and must be set for the session to be created
        W3C Spec capabilities.alwaysMatch must be set for session to be created; 
        capabilities.firstMatch must match at least one (the first one to match will be used)

        useage:
        desired_caps = {
            'platformName': 'Android',
            'platformVersion': '7.0',
            'deviceName': 'Android Emulator',
            'automationName': 'UiAutomator2',
            'app': PATH('/path/to/app')
            }
        self.driver = webdriver.Remote('http://127.0.0.1:4723/wd/hub', desired_caps)

        '''

    def create_NewSession(self, hostAddress: str, desired_caps: Dict[str, str]):
        self.driver = webdriver.Remote(hostAddress, desired_caps)
        '''
        end the session
        '''

    def end_Session(self):
        self.driver.quit()
        ''' Returns dict of capabilities '''

    def get_Session_Capabilities(self):
        desired_caps = self.driver.session
        ''' 
        Navigate backwards in the driver history, if possible (Web context only)

        '''

    def go_Back(self):
        self.driver.back()
        '''
        Take a screenshot of the current viewport/window/page
        Takes a screenshot of the viewport in a native context (iOS, Android) and 
        takes a screenshot of the window in web context

        Note that some platforms may have settings that prevent screenshots from being taken, 
        for security reason. One such feature is the Android FLAG_SECURE layout parameter


        '''

    def take_screenshot(self):
        screenshotBase64 = self.driver.get_screenshot_as_base64()
        return screenshotBase64

        '''
        Get the current application hierarchy XML (app) or page source (web)
        In a web context, the source returns the source HTML of the current window. 
        In a native context (iOS, Android, etc...) it will return the application hierarchy XML.

        This method is useful for inspecting your application hierarchy and using that to write selectors

        (NOTE: iOS and Android don't have standard ways of defining their application source, 
        so on calls to 'Get Page Source' Appium traverses the app hierarchy and creates an XML document. 
        Thus, getting the source can often be an expensive and time-consuming operation)


        '''

    def get_PageSource(self):
        source = self.driver.page_source
        return source
        '''
        Configure the amount of time that a particular type of operation can execute for before they are aborted


        '''

    def set_PageLoad_Timeout(self, timeout: int):
        self.driver.set_page_load_timeout(timeout)

        '''Set the amount of time, in milliseconds, that asynchronous scripts executed by execute async are 
        permitted to run before they are aborted (Web context only)
        
        '''

    def set_scriptLoad_Timeout(self, timeout: int):
        self.driver.set_script_timeout(timeout)
        '''
        Set the amount of time the driver should wait when searching for elements
        '''

    def set_Implicit_Wait_Timeout(self, timeout: int):
        self.driver.implicitly_wait(5)  # waits 5 seconds
        '''
        Inject a snippet of JavaScript into the page for execution in the context of the currently 
        selected frame (Web context only)
        
        Args:	
        script: The JavaScript to execute.
        *args: Any applicable arguments for your JavaScript.
        Usage:	
        script = "var callback = arguments[arguments.length - 1]; " \
                "window.setTimeout(function(){ callback('timeout') }, 3000);"
        driver.execute_async_script(script
        '''

    def execute_Async_Script(self, script: str, *args):
        self.driver.execute_async_script(script)

        '''
        execute_script(script, *args)¶
        Synchronously Executes JavaScript in the current window/frame.

        Args:	
        script: The JavaScript to execute.
        *args: Any applicable arguments for your JavaScript.
        Usage:	
        driver.execute_script('return document.title;')
        '''

    def execute_Script(self, script: str, *args):
        self.driver.execute_script(script)
        '''
        Get the current device/driver orientation
        '''

    def get_Orientation(self):
        orientation = self.driver.orientation
        return orientation
        '''
        Set the current device/driver orientation
        '''

    def set_Orientation(self, orientation: str):
        self.driver.orientation = orientation

        '''
        Get the current geo location
        '''

    def get_geoLocation(self):
        location = self.driver.location()
        return location

    def find_elementBy(self, locatorType, locator):
        locatorType=locatorType
        locator=locator        
        return mobAppium.__with_findElement(self,locatorType, locator)

    def find_elementBy(self, locatorType: str, locator: any):
        locatorType=locatorType
        locator=locator
        print (mobAppium.__with_findElements(self,locatorType, locator))
        return mobAppium.__with_findElements(self,locatorType, locator)

    def __with_findElements(self, locatorType, locator):

        if locatorType == "XPATH":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.XPATH, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "ANDROID_UIAUTOMATOR":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_UIAUTOMATOR, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "ANDROID_VIEWTAG":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_VIEWTAG, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "ANDROID_DATA_MATCHER":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_DATA_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "ANDROID_VIEW_MATCHER":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_VIEW_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "ACCESSIBILITY_ID":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ACCESSIBILITY_ID, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception
        if locatorType == "IMAGE":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.IMAGE, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception:
                return Exception

    def __with_findElement(self, locatorType, locator):

        if locatorType == "XPATH":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.XPATH, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                
                return element
            except Exception:
                #print(Exception,'--------------------')
                return Exception
        if locatorType == "ANDROID_UIAUTOMATOR":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_UIAUTOMATOR, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception:
                return Exception
        if locatorType == "ANDROID_VIEWTAG":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_VIEWTAG, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception:
                return Exception
        if locatorType == "ANDROID_DATA_MATCHER":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_DATA_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception:
                return Exception
        if locatorType == "ANDROID_VIEW_MATCHER":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_VIEW_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=5, timeout=30)
                return element
            except Exception:
                return Exception
        if locatorType == "ACCESSIBILITY_ID":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ACCESSIBILITY_ID, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception:
                return Exception
        if locatorType == "IMAGE":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.IMAGE, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception:
                return Exception



import cv2
import numpy as np
import easyocr
import matplotlib.pyplot as pl
from PIL import Image, ImageChops
from pyallied.web.webWaits import customwebDriverwait
import textract
import os
import slate3k
import PyPDF2
from imap_tools import MailBox,AND
class miscellaneous(customwebDriverwait):
    def __init__(self,driver):
        super().__init__(driver)
        self.driver=driver    
    '''
    returns the embedded text in the image.
    It returns a list of detected text, with each text element containing three types of information. 
    Which are: the text, its bounding box vertices, and the confidence level of the text detection
    '''    
    def get_embeddedText_from_image(self,path:str):
        try:
            reader= easyocr.Reader(['en'])
            return reader.readtext(path)
        except Exception as error:
            raise error
    def is_images_same_compare_by_pixel(self,sourceFile, destinationFile):
        try:
            diff_img=ImageChops.difference(Image.open(sourceFile),Image.open(destinationFile)).convert('RGB')
            if diff_img.getbbox():
                return False
            else:
                return True
        except Exception as error:
            raise error        
    def pdf_content_reader(self,file1):
        try:
            file = open(file1,'rb')
          
            pdfRead=PyPDF2.PdfFileReader(file)
            #print(pdfRead.getDocumentInfo().title,' ----title----')
            output = []    
            readData=pdfRead.getPage(0)
            #print(readData.extractText(),' ----single----')
            for page in pdfRead.pages:
                output.append(page.extractText().replace("\n",""))            
            file.close()
            return output            
        except Exception as error:
            raise error
    def mail_outlook_search(self,userName,password,searchString:str,fromAddress:str):
        try:
            mailbox = MailBox('https://outlook.office.com/',993)
            mailbox.login(userName, password,'INBOX')
            responses =mailbox.idle.wait(timeout=120)
            mailText=''
            if responses:
                for msg in mailbox.fetch(AND(text=searchString,new=True,from_=fromAddress)):
                    mailText=msg.text
                    break
            return mailText
        except Exception as error:
            raise error
    def mail_outlook_Attachments(self,userName,password,searchString:str,fromAddress:str):
        try:
            mailbox = MailBox('https://outlook.office.com/',993)
            mailbox.login(userName, password,'INBOX')
            responses =mailbox.idle.wait(timeout=120)
            attachment=[]
            if responses:
                for msg in mailbox.fetch(AND(text=searchString,new=True,from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment.append(attach.filename)
            return attachment                                
        except Exception as error:
            raise error                            
    def mail_outlook_Attachments_Read(self,userName,password,searchString:str,fromAddress:str):
        try:
            mailbox = MailBox('https://outlook.office.com/',993)
            mailbox.login(userName, password,'INBOX')
            responses =mailbox.idle.wait(timeout=120)
            attachment_Read={}
            if responses:
                for msg in mailbox.fetch(AND(text=searchString,new=True,from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment_Read[attach.filename]=attach.payload
                        #attachment_Read.append(attach.payload)
            return attachment_Read                           
        except Exception as error:
            raise error  
